import { NextRequest, NextResponse } from "next/server";
import { getToken } from "next-auth/jwt";

export async function middleware(req: NextRequest) {
  const token = await getToken({ req });
  const { pathname } = req.nextUrl;

  // Redirect to signin if the user is not authenticated
  if (!token && (pathname.startsWith('/admin') || pathname.startsWith('/officer'))) {
    // Construct the URL to redirect to signin
    const signInUrl = new URL('/auth/signin', req.nextUrl.origin);
    return NextResponse.redirect(signInUrl);
  }

  // Redirect to admin if the user is authenticated 
  if (token && pathname === '/auth/signin') {
    // Type assertion to ensure roles is recognized as a string array
    const roles = (token.roles as string[]) || [];

    if (roles.includes("Administrator") || roles.includes("IBC Manager") || roles.includes("Project Admin")) {
      const adminUrl = token?.projects?.length > 1 ? new URL("/admin/dashboard?projects=true", req.nextUrl.origin) : new URL("/admin/dashboard", req.nextUrl.origin);
      return NextResponse.redirect(adminUrl);
    } else {
      const officerUrl = new URL("/officer/dashboard", req.nextUrl.origin);
      return NextResponse.redirect(officerUrl);
    }
  }

  // Continue with the request if no redirection is needed
  return NextResponse.next();
}

export const config = {
  matcher: ["/admin/:path*", "/officer/:path*", "/auth/:path*"]
};
